@@ -6,9 +6,10 @@ import pprint
 import random
 import string
 import allure
+import zipfile
 import json
 import unittest
-import psutil
+import requests
 from selenium import webdriver
 from selenium.common.exceptions import NoSuchElementException, NoAlertPresentException, NoSuchWindowException, \
     TimeoutException, StaleElementReferenceException, ElementClickInterceptedException, \
@@ -74,6 +75,7 @@ class Selenium(Functions, Parameters):
     pid = None
     record = None
 
+
     # INICIALIZA LOS DRIVER Y LOS CONFIGURA
     def open_browser(self, url=None, browser=Parameters.browser, options_headless=Parameters.headless,
                      download_path=None, record=None, privated=True):
@@ -92,9 +94,12 @@ class Selenium(Functions, Parameters):
                     -El directorio base
                     -El navegador utilizado
         """
-        print(f"Directorio Base: {Parameters.current_path}")
-        print(f"{self.color_message('YELLOW', 'AGUARDANDO:')} Inicio del navegador "
-              f"'{self.color_message('BLUE', browser)}'.")
+        #print(f"Directorio Base: {Parameters.current_path}")
+        Functions.LoggingFeature(f"Directorio Base: {Parameters.current_path}").send_log()
+        Functions.LoggingFeature(f"{self.color_message('YELLOW', 'AGUARDANDO:')} Inicio del navegador "
+              f"'{self.color_message('BLUE', browser)}'.").send_log()
+        #print(f"{self.color_message('YELLOW', 'AGUARDANDO:')} Inicio del navegador "
+              #f"'{self.color_message('BLUE', browser)}'.")
         options = None
         if browser == "CHROME":
             options = webdriver.ChromeOptions()
@@ -124,8 +129,8 @@ class Selenium(Functions, Parameters):
         options.add_argument("--disable-extensions")
         options.add_argument("--dns-prefetch-disable")
         options.add_argument("--verbose")
-        options.add_argument("--disable-popup-blocking")
-        options.add_argument("--proxy-server='direct://'")
+        #options.add_argument("--disable-popup-blocking")
+        #options.add_argument("--proxy-server='direct://'")
         options.add_argument("--proxy-bypass-list=*")
         options.add_argument("--disable-gpu") if os.name == "nt" else None
         if Parameters.environment == "Linux":
@@ -138,8 +143,7 @@ class Selenium(Functions, Parameters):
                 Selenium.driver.maximize_window()
 
         if Parameters.environment == "Windows":
-            if Parameters.environment_configuration == "standalone":
-                options.add_argument(f"--remote-debugging-port={Selenium.available_port()}")
+            if Parameters.environment_configuration == "standalone" or Parameters.environment_configuration is None:
                 Selenium.set_mode_debug(False)
 
             if options_headless or Parameters.environment_configuration == "server":
@@ -194,12 +198,17 @@ class Selenium(Functions, Parameters):
 
         try:
             if browser == "CHROME":
+                URL = "https://storage.googleapis.com/chrome-for-testing-public/124.0.6367.155/win64/chrome-win64.zip"
+                PATH_CHROME_TEMP = "chromium_windows_x64.temp"
                 if Parameters.environment_configuration == "server":
-                    custom_chrome_path = 'C:\Chromium\chrome.exe'
-                    options.binary_location = custom_chrome_path
-                    Selenium.driver = webdriver.Chrome(service=self.chrome_services, options=options)
+                    options.binary_location =  r'C:\\Chromium\\chrome.exe'
                 else:
-                    Selenium.driver = webdriver.Chrome(service=self.chrome_services, options=options)
+                    if not os.path.exists("C:/ChromeForTesting/chrome-win64/chrome.exe"):
+                        Selenium.download_chromium(URL, PATH_CHROME_TEMP)
+                        Selenium.unzip_and_rename(PATH_CHROME_TEMP, "C:/ChromeForTesting/")
+                        os.remove(PATH_CHROME_TEMP)
+                    options.binary_location = "C:/ChromeForTesting/chrome-win64/chrome.exe"
+                Selenium.driver = webdriver.Chrome(service=self.chrome_services, options=options)
 
             elif browser == "FIREFOX":
                 Selenium.driver = webdriver.Firefox(service=self.firefox_services, options=options)
@@ -209,13 +218,27 @@ class Selenium(Functions, Parameters):
 
             elif browser == "EDGE":
                 Selenium.driver = webdriver.Edge(service=self.edge_services, options=options)
-            Selenium.pid = Selenium.driver.service.process.pid
+            #Selenium.pid = Selenium.driver.service.process.pid
 
         except Exception as e:
             Functions.exception_logger(e)
             Selenium.tear_down(self)
             unittest.TestCase().skipTest(f"El web driver no esta disponible para esta prueba. {e}")
 
+
+    @staticmethod
+    def download_chromium(url, save_path):
+        with requests.get(url, stream=True) as response:
+            response.raise_for_status()
+            with open(save_path, 'wb') as f:
+                for chunk in response.iter_content(chunk_size=8192):
+                    f.write(chunk)
+
+    @staticmethod
+    def unzip_and_rename(zip_file, destination_folder):
+        with zipfile.ZipFile(zip_file, 'r') as zip_ref:
+            zip_ref.extractall(destination_folder)
+
     def tear_down(self):
 
         """
@@ -228,18 +251,18 @@ class Selenium(Functions, Parameters):
                 print("====================Inicio Cache===================")
                 pprint.pprint(Selenium.data_cache)
                 print("=====================Fin Cache=====================")
-            print(f"{Selenium.color_message('YELLOW', 'AGUARDANDO:')} Se cerrará el web driver.")
+            Functions.LoggingFeature(f"{Selenium.color_message('YELLOW', 'AGUARDANDO:')} Se cerrará el web driver.").send_log()
+            #print(f"{Selenium.color_message('YELLOW', 'AGUARDANDO:')} Se cerrará el web driver.")
             if Selenium.driver is not None:
+                Selenium.driver.close()
                 Selenium.driver.quit()
-                process = psutil.Process(Selenium.pid)
-                process.terminate()
-
 
         except Exception as e:
             Functions.exception_logger(e)
 
         finally:
-            print(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Finaliza la ejecución.")
+            #print(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Finaliza la ejecución.")
+            Functions.LoggingFeature(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Finaliza la ejecución.").send_log()
             if Selenium.record:
                 self.screen_recorder.stop_recording()
 
@@ -316,7 +339,7 @@ class Selenium(Functions, Parameters):
         """
 
         Selenium.driver.refresh()
-        Selenium.page_has_loaded()
+        Selenium.page_has_loaded(self)
 
     def get_proyect_name(self):
 
@@ -360,30 +383,30 @@ class Selenium(Functions, Parameters):
         elements = None
         try:
             elements = Selenium.driver.find_element(type_locator, indentify)
-            print(f"Se interactuo con el elemento {indentify}")
-            print(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Se detecto el elemento web "
+            Functions.LoggingFeature(f"Se interactuo con el elemento {indentify}").send_log()
+            Functions.LoggingFeature(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Se detecto el elemento web "
                   f"'{Selenium.color_message('BLUE', entity)}' utilizando el "
-                  f"identificador {type_locator} apuntando a '{indentify}'")
+                  f"identificador {type_locator} apuntando a '{indentify}'").send_log()
             find_element = True
 
         except NoSuchElementException:
             Selenium.exception = "NoSuchElementException"
-            print(f"No se pudo encontrar el elemento web '{Selenium.color_message('BLUE', entity)}'"
-                  f" utilizando el identificador {type_locator} apuntando a '{indentify}'")
+            Functions.LoggingFeature(f"No se pudo encontrar el elemento web '{Selenium.color_message('BLUE', entity)}'"
+                  f" utilizando el identificador {type_locator} apuntando a '{indentify}'").send_log()
             Selenium.message_error = f"No se pudo encontrar el elemento web '{entity}' utilizando el identificador " \
                                      f"{type_locator} apuntando a '{indentify}'"
 
         except TimeoutException:
             Selenium.exception = "TimeoutException"
-            print(f"Se agoto el tiempo de busqueda intentando encontrar el elemento web "
+            Functions.LoggingFeature(f"Se agoto el tiempo de busqueda intentando encontrar el elemento web "
                   f"'{Selenium.color_message('BLUE', entity)}' utilizando el identificador"
-                  f" {type_locator} apuntando a '{indentify}'")
+                  f" {type_locator} apuntando a '{indentify}'").send_log()
             Selenium.message_error = f"Se agoto el tiempo de busqueda intentando encontrar el elemento web '{entity}'" \
                                      f" utilizando el identificador " \
                                      f"{type_locator} apuntando a '{indentify}'"
         except Exception as e:
             Functions.exception_logger(e)
-            print(f"Ha ocurrido un error inesperado en tiempo de ejecución.")
+            Functions.LoggingFeature(f"Ha ocurrido un error inesperado en tiempo de ejecución.").send_log()
         Selenium.lista_descripcion_pasos.append(indentify)
         return elements, find_element
 
@@ -432,7 +455,7 @@ class Selenium(Functions, Parameters):
         """
 
         element = None
-        Selenium.page_has_loaded()
+        Selenium.page_has_loaded(self)
         get_entity = Selenium.get_entity(self, entity)
         if get_entity is None:
             print("No se encontro el value en el Json definido.")
@@ -479,8 +502,10 @@ class Selenium(Functions, Parameters):
                         Functions.get_json_file(self, Functions.json_on_loaded)
                         Functions.set_retry(self, 3)
                         return Functions.capture_element(self, entity)
-                Selenium.screenshot(self, "Ultima screenshot antes de finalizar la ejecución")
-                unittest.TestCase().fail(f"--{Selenium.exception}-- El objeto {entity} no se visualiza en pantalla.")
+            Selenium.screenshot(self, "Ultima screenshot antes de finalizar la ejecución")
+            Selenium.tear_down(self)
+            unittest.TestCase().fail(f"--{Selenium.exception}-- El objeto {entity} no se visualiza en pantalla. "
+                                     f"REF:{Selenium.value_to_find}")
 
         return element
 
@@ -499,7 +524,7 @@ class Selenium(Functions, Parameters):
         """
 
         element = Selenium.capture_element(self, entity, variable_y=variable_y, variable_x=variable_x)
-        Selenium.page_has_loaded()
+        Selenium.page_has_loaded(self)
         return ElementUI(element, Selenium.driver, Selenium.value_to_find, Selenium.get_locator_type, entity)
 
     def debugger(self, debug_this_entity):
@@ -551,13 +576,19 @@ class Selenium(Functions, Parameters):
         try:
             with open(json_path, "r", encoding='utf8') as read_file:
                 Selenium.json_strings = json.loads(read_file.read())
-                print(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Se a cargado el respositorio de objetos "
+                Functions.LoggingFeature(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Se a cargado el respositorio de objetos "
                       f"'{Selenium.color_message('BLUE', f'{file}.json')}' encontrado en el directorio "
-                      f"'{json_path}'.")
+                      f"'{json_path}'.").send_log()
+                #print(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Se a cargado el respositorio de objetos "
+                      #f"'{Selenium.color_message('BLUE', f'{file}.json')}' encontrado en el directorio "
+                      #f"'{json_path}'.")
         except FileNotFoundError:
             Selenium.json_strings = False
-            print(f"{Selenium.color_message('RED', 'ERROR:')} No se encontro "
-                  f"'{Selenium.color_message('BLUE', f'{file}.json')}' en el directorio '{json_path}'.")
+            Functions.LoggingFeature(
+                f"{Selenium.color_message('RED', 'ERROR:')} No se encontro "
+                  f"'{Selenium.color_message('BLUE', f'{file}.json')}' en el directorio '{json_path}'.").send_log()
+            #print(f"{Selenium.color_message('RED', 'ERROR:')} No se encontro "
+                  #f"'{Selenium.color_message('BLUE', f'{file}.json')}' en el directorio '{json_path}'.")
             unittest.TestCase().skipTest(f"get_json_file: No se encontro el Archivo {file}")
             Selenium.tear_down(self)
 
@@ -696,7 +727,7 @@ class Selenium(Functions, Parameters):
             Selenium.driver.switch_to.window(Selenium.windows[window])
             Selenium.driver.maximize_window()
             print("Estas en " + window + " : " + Selenium.windows[window])
-            Selenium.page_has_loaded()
+            Selenium.page_has_loaded(self)
 
     def close_page(self):
 
@@ -729,10 +760,9 @@ class Selenium(Functions, Parameters):
         """
 
         Selenium.driver.execute_script(f'''window.open("{url}","_blank");''')
-        Selenium.page_has_loaded()
+        Selenium.page_has_loaded(self)
 
-    @staticmethod
-    def page_has_loaded():
+    def page_has_loaded(self):
 
         """
             Description:
@@ -742,7 +772,7 @@ class Selenium(Functions, Parameters):
         """
 
         try:
-            WebDriverWait(Selenium.driver, 60).until(
+            WebDriverWait(Selenium.driver, 30).until(
                 lambda target: Selenium.driver.execute_script('return document.readyState;') == 'complete')
 
 
@@ -754,6 +784,7 @@ class Selenium(Functions, Parameters):
             except Exception as e:
                 Functions.exception_logger(e)
                 print(f"No se pudo realizar la screenshot de pantalla.")
+            Selenium.tear_down(self)
             unittest.TestCase().fail("--TimeoutException-- No se ha podido realizar la carga de la página.")
 
     def scroll_to(self, locator, y=None, x=None):
@@ -1237,7 +1268,7 @@ class Selenium(Functions, Parameters):
             Returns:
                 Retorna la imágen y descripción de la screenshot de pantalla.
         """
-        Selenium.page_has_loaded()
+        Selenium.page_has_loaded(self)
         try:
             allure.attach(Selenium.driver.get_screenshot_as_png(), description,
                           attachment_type=allure.attachment_type.PNG)
@@ -2054,7 +2085,8 @@ class ElementUI(Selenium):
                     self.retry += 1
                     self.exception = "StaleElementReferenceException"
                     self.message = f'--{self.exception}-- No se ha podido {self.message} ' \
-                                   f'debido a que el objeto "{self.entity}" a sido actualizado repentinamente.'
+                                   f'debido a que el objeto "{self.entity}" a sido actualizado repentinamente. ' \
+                                   f'REF:{self.json_ValueToFind }'
                     Selenium.message_error = self.message
                     Selenium.exception = self.exception
 
@@ -2062,7 +2094,8 @@ class ElementUI(Selenium):
                     self.exception = "ElementClickInterceptedException"
                     self.retry = Parameters.number_retries
                     self.message = f'--{self.exception}-- No se ha podido {self.message} ' \
-                                   f'debido a que el objeto "{self.entity}" se encuentra solapado por otro objeto.'
+                                   f'debido a que el objeto "{self.entity}" se encuentra solapado por otro objeto. ' \
+                                   f'REF:{self.json_ValueToFind}'
                     Selenium.message_error = self.message
                     Selenium.exception = self.exception
 
@@ -2070,7 +2103,8 @@ class ElementUI(Selenium):
                     self.retry += 1
                     self.exception = "ElementNotInteractableException"
                     self.message = f'--{self.exception}-- No se ha podido {self.message} ' \
-                                   f'debido a que el objeto "{self.entity}" no esta disponible.'
+                                   f'debido a que el objeto "{self.entity}" no esta disponible. ' \
+                                   f'REF:{self.json_ValueToFind}'
                     Selenium.message_error = self.message
                     Selenium.exception = self.exception
 
@@ -2078,7 +2112,8 @@ class ElementUI(Selenium):
                     self.retry += 1
                     self.exception = "NoSuchElementException"
                     self.message = f'--{self.exception}-- No se ha podido {self.message} ' \
-                                   f'debido a que el objeto "{self.entity}" no esta disponible.'
+                                   f'debido a que el objeto "{self.entity}" no esta disponible. ' \
+                                   f'REF:{self.json_ValueToFind}'
                     Selenium.message_error = self.message
                     Selenium.exception = self.exception
 
@@ -2086,7 +2121,8 @@ class ElementUI(Selenium):
                     self.retry += 1
                     self.exception = type(e).__name__
                     self.message = f'--{self.exception}-- No se ha podido {self.message} ' \
-                                   f'debido a que ha ocurrido un error inesperado.'
+                                   f'debido a que ha ocurrido un error inesperado. ' \
+                                   f'REF:{self.json_ValueToFind}'
                     Selenium.message_error = self.message
                     Selenium.exception = self.exception
 
@@ -2102,8 +2138,8 @@ class ElementUI(Selenium):
                         self.message = f"--{self.exception}-- {self.message}"
                     unittest.TestCase().fail(self.message)
 
-        print(f"{Functions.color_message('GREEN', 'REALIZADO:')} Se pudo {self.message} sobre "
-              f"'{Functions.color_message('BLUE', self.entity)}'.")
+        Functions.LoggingFeature(f"{Functions.color_message('GREEN', 'REALIZADO:')} Se pudo {self.message} sobre "
+              f"'{Functions.color_message('BLUE', self.entity)}'.").send_log()
         Selenium.message_container = self.message
         Selenium.lista_pasos.append(action)
         if out is not None:
@@ -2364,8 +2400,9 @@ class ScreenRecord(Selenium):
             @returns:
                 None
         """
+
         list_of_files = list(filter(os.path.isfile, glob.glob(img_folder + '*.png')))
-        list_of_files.sort(key=lambda f: int(re.sub('\D', '', f)))
+        list_of_files.sort(key=lambda f: int(re.sub(r"\\D", '', f)))
         if list_of_files:
             im = Image.open(list_of_files[0])
             width, height = im.size
