@@ -18,6 +18,9 @@ class MyTestCase(Selenium, unittest.TestCase):
         valor_variable = os.getenv("PYBOT_SYSTEM")
         print("El valor de la variable de entorno es:", valor_variable)
 
+    def test_003_errores(self):
+        self.flujo_basico2()
+
     def flujo_basico(self):
         Selenium.json_strings = {
             "<textarea>_1331": {
@@ -30,6 +33,23 @@ class MyTestCase(Selenium, unittest.TestCase):
         Selenium.get_element(Selenium, "<textarea>_1331").send_keys("testing de prueba")
         Selenium.get_element(Selenium, "<textarea>_1331").send_special_key("ENTER")
 
+    def flujo_basico2(self):
+        Selenium.json_strings = {
+            "<textarea>_1331": {
+                "GetFieldBy": "Xpath",
+                "ValueToFind": "//textarea[@id=\"APjFqb\"]",
+                "Frame": "ROOT"
+            },
+            "<input>_1331": {
+                "GetFieldBy": "Xpath",
+                "ValueToFind": "/html/body/div[1]/div[3]/form/div[1]/div[1]/div[4]/center/input[1]",
+                "Frame": "ROOT"
+            }
+        }
+        Selenium.open_browser(Selenium, "https://www.google.com/", browser='CHROME', privated=False)
+        Selenium.get_element(Selenium, "<textarea>_1331").send_keys("testing de prueba")
+        Selenium.get_element(Selenium, "<input>_1331").click()
+
     def tearDown(self) -> None:
         Selenium.tear_down(self)
 
